# Copyright (c) 2017 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.

from unittest import mock

from rhsmlib.services import consumer

from subscription_manager import injection as inj
from subscription_manager.identity import Identity
from test.rhsmlib.base import InjectionMockingTest


class TestConsumerService(InjectionMockingTest):
    def setUp(self):
        super(TestConsumerService, self).setUp()
        self.mock_identity = mock.Mock(spec=Identity, name="Identity").return_value
        self.mock_identity.is_valid.return_value = True
        self.mock_identity.uuid = "43b30b32-86cf-459e-9310-cb4182c23c4a"

    def injection_definitions(self, *args, **kwargs):
        if args[0] == inj.IDENTITY:
            return self.mock_identity
        else:
            return None

    def test_get_consumer_uuid(self):
        """
        Test of getting UUID
        """
        test_concumer = consumer.Consumer()
        uuid = test_concumer.get_consumer_uuid()
        self.assertEqual(uuid, "43b30b32-86cf-459e-9310-cb4182c23c4a")

    def test_get_consumer_uuid_unregistered_system(self):
        """
        When system is not registered, then get_consumer_uuid should
        return empty string.
        :return:
        """
        self.mock_identity.uuid = None
        test_concumer = consumer.Consumer()
        uuid = test_concumer.get_consumer_uuid()
        self.assertEqual(uuid, "")



#
# Subscription manager command line utility.
#
# Copyright (c) 2021 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
import logging
import os

import rhsm.connection as connection
import subscription_manager.injection as inj

from rhsmlib.facts.hwprobe import ClassicCheck

from subscription_manager import managerlib
from subscription_manager.branding import get_branding
from subscription_manager.cli import system_exit
from subscription_manager.cli_command.cli import handle_exception
from subscription_manager.cli_command.environments import MULTI_ENV
from subscription_manager.cli_command.user_pass import UserPassCommand
from subscription_manager.i18n import ugettext as _
from subscription_manager.i18n import ungettext
from subscription_manager.utils import get_current_owner, get_supported_resources

log = logging.getLogger(__name__)


class IdentityCommand(UserPassCommand):
    def __init__(self):
        shortdesc = _("Display the identity certificate for this system or " "request a new one")

        super(IdentityCommand, self).__init__("identity", shortdesc, False)

        self.parser.add_argument(
            "--regenerate",
            action="store_true",
            help=_("request a new certificate be generated"),
        )
        self.parser.add_argument(
            "--force",
            action="store_true",
            help=_(
                "force certificate regeneration (requires username and password); "
                "Only used with --regenerate"
            ),
        )

    def _validate_options(self):
        self.assert_should_be_registered()
        if self.options.force and not self.options.regenerate:
            system_exit(os.EX_USAGE, _("--force can only be used with --regenerate"))
        if (self.options.username or self.options.password) and not self.options.force:
            system_exit(os.EX_USAGE, _("--username and --password can only be used with --force"))
        if self.options.token and not self.options.force:
            system_exit(os.EX_USAGE, _("--token can only be used with --force"))

    def _do_command(self):
        # get current consumer identity
        identity = inj.require(inj.IDENTITY)

        # check for Classic before doing anything else
        if ClassicCheck().is_registered_with_classic():
            if identity.is_valid():
                print(_("server type: {type}").format(type=get_branding().REGISTERED_TO_BOTH_SUMMARY))
            else:
                # no need to continue if user is only registered to Classic
                print(_("server type: {type}").format(type=get_branding().REGISTERED_TO_OTHER_SUMMARY))
                return

        try:
            self._validate_options()
            consumerid = self.identity.uuid
            consumer_name = self.identity.name
            if not self.options.regenerate:
                owner = get_current_owner(self.cp, self.identity)
                ownername = owner["displayName"]
                ownerid = owner["key"]

                print(_("system identity: {consumerid}").format(consumerid=consumerid))
                print(_("name: {consumer_name}").format(consumer_name=consumer_name))
                print(_("org name: {ownername}").format(ownername=ownername))
                print(_("org ID: {ownerid}").format(ownerid=ownerid))

                supported_resources = get_supported_resources(self.cp, self.identity)
                if "environments" in supported_resources:
                    consumer = self.cp.getConsumer(consumerid)
                    evn_key = "environments" if self.cp.has_capability(MULTI_ENV) else "environment"
                    environments = consumer[evn_key]
                    if environments:
                        if evn_key == "environment":
                            environment_names = environments["name"]
                        else:
                            environment_names = ",".join(
                                [environment["name"] for environment in environments]
                            )
                    else:
                        environment_names = _("None")
                    print(
                        ungettext(
                            "environment name: {environment_name}",
                            "environment names: {environment_name}",
                            len(environment_names.split(",")),
                        ).format(environment_name=environment_names)
                    )
            else:
                if self.options.force:
                    # get an UEP with basic auth or keycloak auth
                    if self.options.token:
                        self.cp = self.cp_provider.get_keycloak_auth_cp(self.options.token)
                    else:
                        self.cp_provider.set_user_pass(self.username, self.password)
                        self.cp = self.cp_provider.get_basic_auth_cp()
                consumer = self.cp.regenIdCertificate(consumerid)
                managerlib.persist_consumer_cert(consumer)

                # do this in persist_consumer_cert? or some other
                # high level, "I just registered" thing
                self.identity.reload()

                print(_("Identity certificate has been regenerated."))

                log.debug("Successfully generated a new identity from server.")
        except connection.GoneException as ge:
            # Gone exception is caught in CliCommand and a consistent message
            # is printed there for all commands
            raise ge
        except connection.RestlibException as re:
            log.exception(re)
            log.error("Error: Unable to generate a new identity for the system: {re}".format(re=re))

            system_exit(os.EX_SOFTWARE, re)
        except Exception as e:
            handle_exception(_("Error: Unable to generate a new identity for the system"), e)



# Copyright (c) 2020 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#

"""
Module for testing AWS part of Python package cloud_what
"""

import unittest
from unittest.mock import patch, Mock
import tempfile
import time

from cloud_what.providers import aws


def send_only_imds_v2_is_supported(request, *args, **kwargs):
    """
    Mock result, when we try to get metadata using GET method against
    AWS metadata provider. This mock is for the case, when only IMDSv2
    is supported by instance.
    :param request: HTTP request
    :return: Mock with result
    """
    mock_result = Mock()

    if request.method == "PUT":
        if request.url == aws.AWSCloudProvider.CLOUD_PROVIDER_TOKEN_URL:
            if "X-aws-ec2-metadata-token-ttl-seconds" in request.headers:
                mock_result.status_code = 200
                mock_result.text = AWS_TOKEN
            else:
                mock_result.status_code = 400
                mock_result.text = "Error: TTL for token not specified"
        else:
            mock_result.status_code = 400
            mock_result.text = "Error: Invalid URL"
    elif request.method == "GET":
        if "X-aws-ec2-metadata-token" in request.headers.keys():
            if request.headers["X-aws-ec2-metadata-token"] == AWS_TOKEN:
                if request.url == aws.AWSCloudProvider.CLOUD_PROVIDER_METADATA_URL:
                    mock_result.status_code = 200
                    mock_result.text = AWS_METADATA
                elif request.url == aws.AWSCloudProvider.CLOUD_PROVIDER_SIGNATURE_URL:
                    mock_result.status_code = 200
                    mock_result.text = AWS_SIGNATURE
                else:
                    mock_result.status_code = 400
                    mock_result.text = "Error: Invalid URL"
            else:
                mock_result.status_code = 400
                mock_result.text = "Error: Invalid metadata token provided"
        else:
            mock_result.status_code = 400
            mock_result.text = "Error: IMDSv1 is not supported on this instance"
    else:
        mock_result.status_code = 400
        mock_result.text = "Error: not supported request method"

    return mock_result


def mock_prepare_request(request):
    return request


AWS_METADATA = """
{
  "accountId" : "012345678900",
  "architecture" : "x86_64",
  "availabilityZone" : "eu-central-1b",
  "billingProducts" : [ "bp-0124abcd", "bp-63a5400a" ],
  "devpayProductCodes" : null,
  "marketplaceProductCodes" : null,
  "imageId" : "ami-0123456789abcdeff",
  "instanceId" : "i-abcdef01234567890",
  "instanceType" : "m5.large",
  "kernelId" : null,
  "pendingTime" : "2020-02-02T02:02:02Z",
  "privateIp" : "12.34.56.78",
  "ramdiskId" : null,
  "region" : "eu-central-1",
  "version" : "2017-09-30"
}
"""

AWS_SIGNATURE = """ABCDEFGHIJKLMNOPQRSTVWXYZabcdefghijklmnopqrstvwxyz01234567899w0BBwGggCSABIIB
73sKICAiYWNjb3VudElkIiA6ICI1NjcwMTQ3ODY4OTAiLAogICJhcmNoaXRlY3R1cmUiIDogIng4
Nl82NCIsCiAgImF2YWlsYWJpbGl0eVpvbmUiIDogImV1LWNlbnRyYWwtMWIiLAogICJiaWxsaW5n
UHJvZHVjdHMiIDogWyAiYnAtNmZhNTQwMDYiIF0sCiAgImRldnBheVByb2R1Y3RDb2RlcyIgOiBu
73sKICAiYWNjb3VudElkIiA6ICI1NjcwMTQ3ODY4OTAiLAogICJhcmNoaXRlY3R1cmUiIDogIng4
bWktMDgxNmFkNzYyOTc2YzM1ZGIiLAogICJpbnN0YW5jZUlkIiA6ICJpLTBkNTU0YzRmM2JhNWVl
YTczIiwKICAiaW5zdGFuY2VUeXBlIiA6ICJtNS5sYXJnZSIsCiAgImtlcm5lbElkIiA6IG51bGws
CiAgInBlbmRpbmdUaW1lIiA6ICIyMDIwLTA0LTI0VDE0OjU3OjQzWiIsCiAgInByaXZhdGVJcCIg
OiAiMTcyLjMxLjExLjc4IiwKICAicmFtZGlza0lkIiA6IG51bGwsCiAgInJlZ2lvbiIgOiAiZXUt
Y2VudHJhbC0xIiwKICAidmVyc2lvbiIgOiAiMjAxNy0wOS0zMCIKfQAAAAAAADGCAf8wggH7AgEB
CiAgInBlbmRpbmdUaW1lIiA6ICIyMDIwLTA0LTI0VDE0OjU3OjQzWiIsCiAgInByaXZhdGVJcCIg
YXR0bGUxIDAeBgNVBAoTF0FtYXpvbiBXZWIgU2VydmljZXMgTExDAgkAoP6/ot5H9aswDQYJYIZI
AWUDBAIBBQCgaTAYBgkqhkiG9w0BCQMxCwYJKoZIhvcNAQcBMBwGCSqGSIb3DQEJBTEPFw0yMDA5
MDExNTMyNDhaMC8GCSqGSIb3DQEJBDEiBCCmizT0hDlJmxHtDBaEjql5ZPFaoKy6OSk7qBFREVRk
iTANBgkqhkiG9w0BAQEFAASCAQAh//5+AaFAcgw/5SoglQ27kQKuThcJYa+QhC2aw4n1GvkvCmyi
helVMxH33tB9tUei/mapSF3v8jUseRLEbcDVRHf6n6h14Qj2MxtgYanzUCDF8qECYbZ2uSy3JLEP
iNsndm8nt7XcJC7NRoWJWAsly1VeXVIauA/l7uXmUarDQs5BhFYl7REX4htxg9mCibR6xqU5i8/D
iTANBgkqhkiG9w0BAQEFAASCAQAh//5+AaFAcgw/5SoglQ27kQKuThcJYa+QhC2aw4n1GvkvCmyi
tGbafapTj+6KnJAfP0sW7ZbzKclaCPHXQ37z9mc8vtCxEQmCbGL6sj2wtpi4rmRlAAAAAAAA"""

AWS_TOKEN = "ABCDEFGHIJKLMNOPQRSTVWXYZabcdefghijklmnopqrstvwxyz0123=="


class TestAWSCloudProvider(unittest.TestCase):
    """
    Class used for testing of AWS cloud provider
    """

    def setUp(self) -> None:
        """
        Destroy instance of singleton and set instance not initialized
        """
        aws.AWSCloudProvider._instance = None
        aws.AWSCloudProvider._initialized = False

    def tearDown(self) -> None:
        """
        Clean after each unit test
        """
        aws.AWSCloudProvider._instance = None
        aws.AWSCloudProvider._initialized = False

    def test_aws_instance_is_singleton(self):
        """
        Any subclass of BaseCloudProvider should behave as singleton
        """
        aws_cloud_provider_01 = aws.AWSCloudProvider({})
        self.assertEqual(aws_cloud_provider_01._initialized, True)
        aws_cloud_provider_02 = aws.AWSCloudProvider({})
        self.assertEqual(id(aws_cloud_provider_01), id(aws_cloud_provider_02))

    def test_aws_cloud_provider_id(self):
        """
        Test of CLOUD_PROVIDER_ID
        """
        self.assertEqual(aws.AWSCloudProvider.CLOUD_PROVIDER_ID, "aws")
        aws_provider = aws.AWSCloudProvider({})
        self.assertEqual(aws_provider.CLOUD_PROVIDER_ID, "aws")

    def test_aws_not_vm(self):
        """
        Test for the case, when the machine is host (not virtual machine)
        """
        # We will mock facts using simple dictionary
        facts = {"virt.is_guest": False, "dmi.bios.version": "cool hardware company"}
        aws_detector = aws.AWSCloudProvider(facts)
        is_vm = aws_detector.is_vm()
        self.assertFalse(is_vm)

    def test_aws_vm_using_xen(self):
        """
        Test for the case, when the vm is running on AWS Xen
        """
        # We will mock facts using simple dictionary
        facts = {"virt.is_guest": True, "virt.host_type": "xen", "dmi.bios.version": "amazon"}
        aws_detector = aws.AWSCloudProvider(facts)
        is_vm = aws_detector.is_vm()
        self.assertTrue(is_vm)
        is_aws_xen_vm = aws_detector.is_running_on_cloud()
        self.assertTrue(is_aws_xen_vm)

    def test_aws_vm_using_kvm(self):
        """
        Test for the case, when the vm is running on AWS KVM
        """
        # We will mock facts using simple dictionary
        facts = {
            "virt.is_guest": True,
            "virt.host_type": "kvm",
            "dmi.bios.version": "1.0",
            "dmi.bios.vendor": "Amazon EC2",
        }
        aws_detector = aws.AWSCloudProvider(facts)
        is_vm = aws_detector.is_vm()
        self.assertTrue(is_vm)
        is_aws_kvm_vm = aws_detector.is_running_on_cloud()
        self.assertTrue(is_aws_kvm_vm)

    def test_vm_not_on_aws_cloud(self):
        """
        Test for the case, when the vm is not running on AWS
        """
        # We will mock facts using simple dictionary
        facts = {
            "virt.is_guest": True,
            "virt.host_type": "kvm",
            "dmi.bios.version": "1.0",
            "dmi.bios.vendor": "Foo",
        }
        aws_detector = aws.AWSCloudProvider(facts)
        is_vm = aws_detector.is_vm()
        self.assertTrue(is_vm)
        is_aws_vm = aws_detector.is_running_on_cloud()
        self.assertFalse(is_aws_vm)

    def test_vm_without_dmi_bios_info(self):
        """
        Test for the case, when SM BIOS does not provide any useful information for our code
        """
        # We will mock facts using simple dictionary
        facts = {}
        aws_detector = aws.AWSCloudProvider(facts)
        is_vm = aws_detector.is_vm()
        self.assertFalse(is_vm)
        is_aws_vm = aws_detector.is_running_on_cloud()
        self.assertFalse(is_aws_vm)

    def test_vm_system_uuid_starts_with_ec2(self):
        """
        Test for the case, when system UUID starts with EC2 string as it is described here:
        https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/identify_ec2_instances.html
        """
        # We will mock facts using simple dictionary
        facts = {"virt.is_guest": True, "dmi.system.uuid": "EC2263F8-15F3-4A34-B186-FAD8AB963431"}
        aws_detector = aws.AWSCloudProvider(facts)
        is_vm = aws_detector.is_vm()
        self.assertTrue(is_vm)
        probability = aws_detector.is_likely_running_on_cloud()
        self.assertEqual(probability, 0.1)

    @patch("cloud_what._base_provider.requests.Session")
    def test_get_metadata_from_server_imds_v1(self, mock_session_class):
        """
        Test the case, when metadata are obtained from server using IMDSv1
        """
        mock_result = Mock()
        mock_result.status_code = 200
        mock_result.text = AWS_METADATA
        mock_session = Mock()
        mock_session.send = Mock(return_value=mock_result)
        mock_session.prepare_request = Mock()
        mock_session.hooks = {"response": []}
        mock_session_class.return_value = mock_session
        aws_collector = aws.AWSCloudProvider({})
        # Mock that no metadata cache exists
        aws_collector._get_metadata_from_cache = Mock(return_value=None)
        metadata = aws_collector.get_metadata()
        self.assertEqual(metadata, AWS_METADATA)

    @patch("cloud_what._base_provider.requests.Session")
    def test_get_signature_from_server_imds_v1(self, mock_session_class):
        """
        Test the case, when metadata are obtained from server using IMDSv1
        """
        mock_result = Mock()
        mock_result.status_code = 200
        mock_result.text = AWS_SIGNATURE
        mock_session = Mock()
        mock_session.send = Mock(return_value=mock_result)
        mock_session.prepare_request = Mock()
        mock_session.hooks = {"response": []}
        mock_session_class.return_value = mock_session

        aws_collector = aws.AWSCloudProvider({})
        # Mock that no metadata cache exists
        aws_collector._get_signature_from_cache = Mock(return_value=None)
        test_signature = aws_collector.get_signature()
        signature = "-----BEGIN PKCS7-----\n" + AWS_SIGNATURE + "\n-----END PKCS7-----"
        self.assertEqual(signature, test_signature)

    @patch("cloud_what._base_provider.requests.Session")
    def test_get_metadata_from_server_imds_v2(self, mock_session_class):
        """
        Test the case, when metadata are obtained from server using IMDSv2
        """
        mock_session = Mock()
        mock_session.send = send_only_imds_v2_is_supported
        mock_session.prepare_request = Mock(side_effect=mock_prepare_request)
        mock_session.hooks = {"response": []}
        mock_session_class.return_value = mock_session

        aws_provider = aws.AWSCloudProvider({})
        # Mock that no metadata cache exists
        aws_provider._get_metadata_from_cache = Mock(return_value=None)
        # Mock that no token cache exists
        aws_provider._get_token_from_cache_file = Mock(return_value=None)
        # Mock writing token to cache file
        aws_provider._write_token_to_cache_file = Mock()
        # Mock getting metadata using IMDSv1 is disabled by user
        aws_provider._get_metadata_from_server_imds_v1 = Mock(return_value=None)

        metadata = aws_provider.get_metadata()
        self.assertEqual(metadata, AWS_METADATA)

    @patch("cloud_what._base_provider.requests.Session")
    def test_get_signature_from_server_imds_v2(self, mock_session_class):
        """
        Test the case, when signature is obtained from server using IMDSv2
        """
        mock_session = Mock()
        mock_session.send = send_only_imds_v2_is_supported
        mock_session.prepare_request = Mock(side_effect=mock_prepare_request)
        mock_session.hooks = {"response": []}
        mock_session_class.return_value = mock_session

        aws_collector = aws.AWSCloudProvider({})
        # Mock that no metadata cache exists
        aws_collector._get_metadata_from_cache = Mock(return_value=None)
        # Mock that no token cache exists
        aws_collector._get_token_from_cache_file = Mock(return_value=None)
        # Mock writing token to cache file
        aws_collector._write_token_to_cache_file = Mock()

        test_signature = aws_collector.get_signature()
        signature = "-----BEGIN PKCS7-----\n" + AWS_SIGNATURE + "\n-----END PKCS7-----"
        self.assertEqual(signature, test_signature)

    @patch("cloud_what._base_provider.requests.Session")
    def test_metadata_in_memory_cache(self, mock_session_class):
        """
        Test that metadata is read from in-memory cache
        """
        # First mock reading metadata from server
        mock_session = Mock()
        mock_session.send = Mock(side_effect=send_only_imds_v2_is_supported)
        mock_session.prepare_request = Mock(side_effect=mock_prepare_request)
        mock_session.hooks = {"response": []}
        mock_session_class.return_value = mock_session

        aws_provider = aws.AWSCloudProvider({})
        # Mock that no metadata cache exists
        aws_provider._get_metadata_from_cache = Mock(return_value=None)
        # Mock that no token cache exists
        aws_provider._get_token_from_cache_file = Mock(return_value=None)
        # Mock writing token to cache file
        aws_provider._write_token_to_cache_file = Mock()
        # Mock getting metadata using IMDSv1 is disabled by user
        aws_provider._get_metadata_from_server_imds_v1 = Mock(return_value=None)

        # Try to get metadata from server
        metadata = aws_provider.get_metadata()

        self.assertEqual(metadata, AWS_METADATA)
        # There should be two calls to IMDS server (put and get)
        self.assertEqual(mock_session.send.call_count, 2)

        # Test that metadata are stored in in-memory cache
        self.assertEqual(metadata, aws_provider._cached_metadata)

        # Try to get metadata once again
        new_metadata = aws_provider.get_metadata()

        # Metadata have to be still the same
        self.assertEqual(new_metadata, AWS_METADATA)
        # There should not be more calls to IMDS server
        # (still only two calls from previous communication)
        self.assertEqual(mock_session.send.call_count, 2)

    def test_reading_valid_cached_token(self):
        """
        Test reading of valid cached token from file
        """
        c_time = str(time.time())
        ttl = str(aws.AWSCloudProvider.CLOUD_PROVIDER_TOKEN_TTL)
        token = "ABCDEFGHy0hY_y8D7e95IIx7aP2bmnzddz0tIV56yZY9oK00F8GUPQ=="
        valid_token = (
            '{\
  "ctime": "'
            + c_time
            + '",\
  "ttl": "'
            + ttl
            + '",\
  "token": "'
            + token
            + '"\
}'
        )
        aws_collector = aws.AWSCloudProvider({})
        # Create mock of cached toke file
        with tempfile.NamedTemporaryFile() as tmp_token_file:
            # Create valid token file
            tmp_token_file.write(bytes(valid_token, encoding="utf-8"))
            tmp_token_file.flush()
            old_token_cache_file = aws_collector.TOKEN_CACHE_FILE
            aws_collector.TOKEN_CACHE_FILE = tmp_token_file.name
            test_token = aws_collector._get_token_from_cache_file()
            self.assertEqual(token, test_token)
            aws_collector.TOKEN_CACHE_FILE = old_token_cache_file

    def test_reading_invalid_cached_token_corrupted_json_file(self):
        """
        Test reading of corrupted cached token from file
        """
        c_time = str(time.time())
        ttl = str(aws.AWSCloudProvider.CLOUD_PROVIDER_TOKEN_TTL)
        token = "ABCDEFGHy0hY_y8D7e95IIx7aP2bmnzddz0tIV56yZY9oK00F8GUPQ=="
        valid_token = (
            '{[\
  "ctime": "'
            + c_time
            + '",\
  "ttl": "'
            + ttl
            + '",\
  "token": "'
            + token
            + '"\
}]'
        )
        aws_collector = aws.AWSCloudProvider({})
        # Create mock of cached toke file
        with tempfile.NamedTemporaryFile() as tmp_token_file:
            # Create valid token file
            tmp_token_file.write(bytes(valid_token, encoding="utf-8"))
            tmp_token_file.flush()
            old_token_cache_file = aws_collector.TOKEN_CACHE_FILE
            aws_collector.TOKEN_CACHE_FILE = tmp_token_file.name
            test_token = aws_collector._get_token_from_cache_file()
            self.assertEqual(test_token, None)
            aws_collector.TOKEN_CACHE_FILE = old_token_cache_file

    def test_reading_invalid_cached_token_wrong_ctime_format(self):
        """
        Test reading of cached token from file with invalid time format
        """
        ttl = str(aws.AWSCloudProvider.CLOUD_PROVIDER_TOKEN_TTL)
        token = "ABCDEFGHy0hY_y8D7e95IIx7aP2bmnzddz0tIV56yZY9oK00F8GUPQ=="
        # ctime has to be float (unix epoch)
        valid_token = (
            '{[\
  "ctime": "Wed Dec 16 16:31:36 CET 2020",\
  "ttl": "'
            + ttl
            + '",\
  "token": "'
            + token
            + '"\
}]'
        )
        aws_collector = aws.AWSCloudProvider({})
        # Create mock of cached toke file
        with tempfile.NamedTemporaryFile() as tmp_token_file:
            # Create valid token file
            tmp_token_file.write(bytes(valid_token, encoding="utf-8"))
            tmp_token_file.flush()
            old_token_cache_file = aws_collector.TOKEN_CACHE_FILE
            aws_collector.TOKEN_CACHE_FILE = tmp_token_file.name
            test_token = aws_collector._get_token_from_cache_file()
            self.assertEqual(test_token, None)
            aws_collector.TOKEN_CACHE_FILE = old_token_cache_file

    def test_reading_invalid_cached_token_missing_key(self):
        """
        Test reading of invalid cached token from file
        """
        valid_token = '{\
  "token": "ABCDEFGHy0hY_y8D7e95IIx7aP2bmnzddz0tIV56yZY9oK00F8GUPQ=="\
}'
        aws_collector = aws.AWSCloudProvider({})
        # Create mock of cached toke file
        with tempfile.NamedTemporaryFile() as tmp_token_file:
            # Create valid token file
            tmp_token_file.write(bytes(valid_token, encoding="utf-8"))
            tmp_token_file.flush()
            old_token_cache_file = aws_collector.TOKEN_CACHE_FILE
            aws_collector.TOKEN_CACHE_FILE = tmp_token_file.name
            test_token = aws_collector._get_token_from_cache_file()
            self.assertEqual(test_token, None)
            aws_collector.TOKEN_CACHE_FILE = old_token_cache_file

    def test_reading_timed_out_cached_token(self):
        """
        Test reading of cached token from file that is timed out
        """
        c_time = str(time.time() - aws.AWSCloudProvider.CLOUD_PROVIDER_TOKEN_TTL - 10)
        ttl = str(aws.AWSCloudProvider.CLOUD_PROVIDER_TOKEN_TTL)
        valid_token = (
            '{\
  "ctime": "'
            + c_time
            + '",\
  "ttl": "'
            + ttl
            + '",\
  "token": "ABCDEFGHy0hY_y8D7e95IIx7aP2bmnzddz0tIV56yZY9oK00F8GUPQ=="\
}'
        )
        aws_collector = aws.AWSCloudProvider({})
        # Create mock of cached toke file
        with tempfile.NamedTemporaryFile() as tmp_token_file:
            # Create valid token file
            tmp_token_file.write(bytes(valid_token, encoding="utf-8"))
            tmp_token_file.flush()
            old_token_cache_file = aws_collector.TOKEN_CACHE_FILE
            aws_collector.TOKEN_CACHE_FILE = tmp_token_file.name
            test_token = aws_collector._get_token_from_cache_file()
            self.assertEqual(test_token, None)
            aws_collector.TOKEN_CACHE_FILE = old_token_cache_file



# Copyright (c) 2017 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
from unittest import mock
import datetime

from test.rhsmlib.base import InjectionMockingTest

from subscription_manager import injection as inj
from subscription_manager.cert_sorter import CertSorter
from subscription_manager.validity import ValidProductDateRangeCalculator

from test import stubs

from rhsm import connection

from rhsmlib.services import products


START_DATE = datetime.datetime.now() - datetime.timedelta(days=100)
NOW_DATE = datetime.datetime.now()
END_DATE = datetime.datetime.now() + datetime.timedelta(days=265)


NO_CONTENT_JSON = [
    {
        "id": "4028fa7a5da1fbc2015da203aba209b7",
        "uuid": "57b7dbff-9489-43ac-991a-b848324b423a",
        "name": "localhost.localdomain",
        "username": "admin",
        "entitlementStatus": "valid",
        "serviceLevel": "",
        "releaseVer": {"releaseVer": None},
        "idCert": {
            "key": "FAKE RSA PRIVATE KEY",
            "cert": "FAKE CERTIFICATE",
            "serial": {
                "id": 8134386700568860251,
                "revoked": False,
                "collected": False,
                "expiration": END_DATE.strftime("%Y-%m-%dT%H:%M:%S+0000"),
                "serial": 8134386700568860251,
                "created": START_DATE.strftime("%Y-%m-%dT%H:%M:%S+0000"),
                "updated": START_DATE.strftime("%Y-%m-%dT%H:%M:%S+0000"),
            },
            "id": "4028fa7a5da1fbc2015da203ad8c09b9",
            "created": START_DATE.strftime("%Y-%m-%dT%H:%M:%S+0000"),
            "updated": START_DATE.strftime("%Y-%m-%dT%H:%M:%S+0000"),
        },
        "type": {"id": "1000", "label": "system", "manifest": False},
        "owner": {
            "id": "4028fa7a5da1fbc2015da1fdb5380004",
            "key": "admin",
            "displayName": "Admin Owner",
            "href": "/owners/admin",
        },
        "environment": None,
        "entitlementCount": 0,
        "facts": {},
        "lastCheckin": NOW_DATE.strftime("%Y-%m-%dT%H:%M:%S+0000"),
        "installedProducts": [],
        "canActivate": False,
        "capabilities": [],
        "hypervisorId": None,
        "contentTags": [],
        "autoheal": True,
        "contentAccessMode": None,
        "recipientOwnerKey": None,
        "annotations": None,
        "href": "/consumers/57b7dbff-9489-43ac-991a-b848324b423a",
        "dev": False,
        "created": START_DATE.strftime("%Y-%m-%dT%H:%M:%S+0000"),
        "updated": START_DATE.strftime("%Y-%m-%dT%H:%M:%S+0000"),
    }
]

CONTENT_JSON = [
    {
        "id": "4028fa7a5da1fbc2015da203aba209b7",
        "uuid": "57b7dbff-9489-43ac-991a-b848324b423a",
        "name": "localhost.localdomain",
        "username": "admin",
        "entitlementStatus": "valid",
        "serviceLevel": "",
        "releaseVer": {"releaseVer": None},
        "idCert": {
            "key": "FAKE RSA PRIVATE KEY",
            "cert": "FAKE CERTIFICATE",
            "serial": {
                "id": 8134386700568860251,
                "revoked": False,
                "collected": False,
                "expiration": END_DATE.strftime("%Y-%m-%dT%H:%M:%S+0000"),
                "serial": 8134386700568860251,
                "created": START_DATE.strftime("%Y-%m-%dT%H:%M:%S+0000"),
                "updated": START_DATE.strftime("%Y-%m-%dT%H:%M:%S+0000"),
            },
            "id": "4028fa7a5da1fbc2015da203ad8c09b9",
            "created": START_DATE.strftime("%Y-%m-%dT%H:%M:%S+0000"),
            "updated": START_DATE.strftime("%Y-%m-%dT%H:%M:%S+0000"),
        },
        "type": {"id": "1000", "label": "system", "manifest": False},
        "owner": {
            "id": "4028fa7a5da1fbc2015da1fdb5380004",
            "key": "admin",
            "displayName": "Admin Owner",
            "href": "/owners/admin",
        },
        "environment": None,
        "entitlementCount": 0,
        "facts": {},
        "lastCheckin": "2017-08-02T08:16:39+0000",
        "installedProducts": [
            {
                "id": "8a99f9895d8b4f96015d9db99e9971fb",
                "productId": "69",
                "productName": "Red Hat Enterprise Linux Server",
                "version": "7.4",
                "arch": "x86_64",
                "status": "green",
                "startDate": START_DATE.strftime("%Y-%m-%dT%H:%M:%S+0000"),
                "endDate": END_DATE.strftime("%Y-%m-%dT%H:%M:%S+0000"),
                "created": START_DATE.strftime("%Y-%m-%dT%H:%M:%S+0000"),
                "updated": START_DATE.strftime("%Y-%m-%dT%H:%M:%S+0000"),
            },
            {
                "id": "8a99f9895d8b4f96015d9db99e9971fc",
                "productId": "70",
                "productName": "Red Hat Enterprise Linux Server - Extended Update Support",
                "version": "7.2",
                "arch": "x86_64",
                "status": "green",
                "startDate": START_DATE.strftime("%Y-%m-%dT%H:%M:%S+0000"),
                "endDate": END_DATE.strftime("%Y-%m-%dT%H:%M:%S+0000"),
                "created": START_DATE.strftime("%Y-%m-%dT%H:%M:%S+0000"),
                "updated": START_DATE.strftime("%Y-%m-%dT%H:%M:%S+0000"),
            },
        ],
        "canActivate": False,
        "capabilities": [],
        "hypervisorId": None,
        "contentTags": [],
        "autoheal": True,
        "contentAccessMode": None,
        "recipientOwnerKey": None,
        "annotations": None,
        "href": "/consumers/57b7dbff-9489-43ac-991a-b848324b423a",
        "dev": False,
        "created": START_DATE.strftime("%Y-%m-%dT%H:%M:%S+0000"),
        "updated": START_DATE.strftime("%Y-%m-%dT%H:%M:%S+0000"),
    }
]


class TestProductService(InjectionMockingTest):
    def setUp(self):
        super(TestProductService, self).setUp()
        self.mock_cert_sorter = mock.Mock(spec=CertSorter, name="CertSorter")
        self.mock_cp = mock.Mock(spec=connection.UEPConnection, name="UEPConnection")
        self.mock_calculator = mock.Mock(
            spec=ValidProductDateRangeCalculator, name="ValidProductDateRangeCalculator"
        )

    def injection_definitions(self, *args, **kwargs):
        if args[0] == inj.CERT_SORTER:
            return self.mock_cert_sorter
        elif args[0] == inj.PRODUCT_DATE_RANGE_CALCULATOR:
            return self.mock_calculator
        else:
            return None

    def _create_rhel74_cert(self):
        return self._create_cert("69", "Red Hat Enterprise Linux Server", "7.4", "rhel-7,rhel-7-server")

    def _create_rhel72_ues_cert(self):
        return self._create_cert(
            "70",
            "Red Hat Enterprise Linux Server - Extended Update Support",
            "7.2",
            "rhel-7-eus-server,rhel-7-server",
        )

    @staticmethod
    def _create_cert(product_id, name, version, provided_tags):
        cert = stubs.StubProductCertificate(
            product=stubs.StubProduct(
                product_id=product_id, name=name, version=version, provided_tags=provided_tags
            ),
            start_date=START_DATE,
            end_date=END_DATE,
        )
        cert.delete = mock.Mock()
        cert.write = mock.Mock()
        return cert

    def test_list_no_installed_products(self):
        self.mock_cp.getConsumer.return_value = NO_CONTENT_JSON
        self.mock_cert_sorter.installed_products = []

        result = products.InstalledProducts(self.mock_cp).list()
        self.assertEqual([], result)

    def test_list_installed_products_without_filter(self):
        self.mock_cp.getConsumer.return_value = CONTENT_JSON
        self.mock_cert_sorter.reasons = mock.Mock()
        self.mock_cert_sorter.reasons.get_product_reasons = mock.Mock(return_value=[])
        self.mock_cert_sorter.get_status = mock.Mock(return_value="subscribed")
        # Mock methods in calculator
        self.mock_calculator.calculate = mock.Mock()

        self.mock_calculator.calculate.return_value.begin = mock.Mock()
        self.mock_calculator.calculate.return_value.begin.return_value.astimezone = mock.Mock()
        self.mock_calculator.calculate.return_value.begin.return_value.astimezone.return_value.strftime = (
            mock.Mock(return_value="{d.day}.{d.month}.{d.year}".format(d=START_DATE))
        )
        self.mock_calculator.calculate.return_value.end = mock.Mock()
        self.mock_calculator.calculate.return_value.end.return_value.astimezone = mock.Mock()
        self.mock_calculator.calculate.return_value.end.return_value.astimezone.return_value.strftime = (
            mock.Mock(return_value="{d.day}.{d.month}.{d.year}".format(d=END_DATE))
        )

        expected_result = [
            (
                "Red Hat Enterprise Linux Server",
                "69",
                "7.4",
                "x86_64",
                "subscribed",
                [],
                "{d.day}.{d.month}.{d.year}".format(d=START_DATE),
                "{d.day}.{d.month}.{d.year}".format(d=END_DATE),
            ),
            (
                "Red Hat Enterprise Linux Server - Extended Update Support",
                "70",
                "7.2",
                "x86_64",
                "subscribed",
                [],
                "{d.day}.{d.month}.{d.year}".format(d=START_DATE),
                "{d.day}.{d.month}.{d.year}".format(d=END_DATE),
            ),
        ]

        self.mock_cert_sorter.installed_products = {
            "69": self._create_rhel74_cert(),
            "70": self._create_rhel72_ues_cert(),
        }

        result = products.InstalledProducts(self.mock_cp).list()

        self.assertEqual(expected_result, result)

    def test_list_installed_products_with_filter(self):
        self.mock_cp.getConsumer.return_value = CONTENT_JSON
        self.mock_cert_sorter.reasons = mock.Mock()
        self.mock_cert_sorter.reasons.get_product_reasons = mock.Mock(return_value=[])
        self.mock_cert_sorter.get_status = mock.Mock(return_value="subscribed")
        # Mock methods in calculator
        self.mock_calculator.calculate = mock.Mock()

        self.mock_calculator.calculate.return_value.begin = mock.Mock()
        self.mock_calculator.calculate.return_value.begin.return_value.astimezone = mock.Mock()
        self.mock_calculator.calculate.return_value.begin.return_value.astimezone.return_value.strftime = (
            mock.Mock(return_value="{d.day}.{d.month}.{d.year}".format(d=START_DATE))
        )
        self.mock_calculator.calculate.return_value.end = mock.Mock()
        self.mock_calculator.calculate.return_value.end.return_value.astimezone = mock.Mock()
        self.mock_calculator.calculate.return_value.end.return_value.astimezone.return_value.strftime = (
            mock.Mock(return_value="{d.day}.{d.month}.{d.year}".format(d=END_DATE))
        )

        expected_result = [
            (
                "Red Hat Enterprise Linux Server - Extended Update Support",
                "70",
                "7.2",
                "x86_64",
                "subscribed",
                [],
                "{d.day}.{d.month}.{d.year}".format(d=START_DATE),
                "{d.day}.{d.month}.{d.year}".format(d=END_DATE),
            ),
        ]

        self.mock_cert_sorter.installed_products = {
            "69": self._create_rhel74_cert(),
            "70": self._create_rhel72_ues_cert(),
        }

        result = products.InstalledProducts(self.mock_cp).list("*Extended*")

        self.assertEqual(expected_result, result)



#!/usr/bin/python

import polib
import sys


class PotFile(object):
    def __init__(self):

        self.msgids = []
        self.po = polib.pofile("po/keys.pot")
        for entry in self.po:
            self.msgids.append(entry.msgid)

        self.msgids.sort()


po = polib.POFile()
po.metadata = {
    "Project-Id-Version": "1.0",
    "Report-Msgid-Bugs-To": "alikins@redhat.com",
    "POT-Creation-Date": "2007-10-18 14:00+0100",
    "PO-Revision-Date": "2007-10-18 14:00+0100",
    "Last-Translator": "gen_test_en_po.py <nothing@example.com>",
    "Language-Team": "English <yourteam@example.com>",
    "MIME-Version": "1.0",
    "Content-Type": "text/plain; charset=utf-8",
    "Content-Transfer-Encoding": "8bit",
}


def gen_msg(msgid, tall=False, longstring=False, wrap=False):
    msg = msgid
    if longstring:
        msg = "%s________________" % msg
    if tall:
        msg = msg + "\n" * 40
    if wrap:
        msg = "xx{%s}" % msg
    return msg


def main():
    args = sys.argv[1:]

    tall = False
    longstring = False
    wrap = False
    if "--tall" in args:
        tall = True
    if "--long" in args:
        longstring = True
    if "--wrap" in args:
        wrap = True

    potfile = PotFile()
    for pot_entry in potfile.po:
        entry = polib.POEntry(
            msgid=pot_entry.msgid,
            msgstr=gen_msg(pot_entry.msgid, tall, longstring, wrap),
            occurrences=pot_entry.occurrences,
        )
        po.append(entry)

    po.save("po/en_US.po")


if __name__ == "__main__":
    main()



# Get the right platform specific provider or a null provider
#
# Copyright (c) 2010-2016 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
import logging
from typing import Dict, Union

from rhsmlib.facts import dmiinfo
from rhsmlib.facts import collector

ARCHES_WITHOUT_DMI = ["ppc64", "ppc64le", "s390x"]

log = logging.getLogger(__name__)


# This doesn't really do anything other than provide a null/noop provider for
# non-DMI platforms.
class NullFirmwareInfoCollector(object):
    """Default provider for platform without a specific platform info provider.

    ie, all platforms except those with DMI (ie, intel platforms)"""

    def __init__(self, *args, **kwargs):
        self.info: Dict = {}

    def get_all(self) -> Dict[str, str]:
        return self.info


class FirmwareCollector(collector.FactsCollector):
    def __init__(
        self,
        prefix: str = None,
        testing: bool = None,
        collected_hw_info: Dict[str, Union[str, int, bool, None]] = None,
    ):
        super(FirmwareCollector, self).__init__(
            prefix=prefix, testing=testing, collected_hw_info=collected_hw_info
        )

    def get_firmware_info(self) -> Dict[str, str]:
        """Read and parse data that comes from platform specific interfaces.

        This is only dmi/smbios data for now (which isn't on ppc/s390).
        """
        firmware_info_collector: Union[NullFirmwareInfoCollector, dmiinfo.DmidecodeFactCollector]
        firmware_info_collector = get_firmware_collector(
            arch=self.arch, prefix=self.prefix, testing=self.testing
        )

        # Pass in collected hardware so DMI etc can potentially override it
        firmware_info_dict: Dict[str, str] = firmware_info_collector.get_all()
        # This can potentially clobber facts that already existed in self.allhw
        # (and is supposed to).
        return firmware_info_dict

    def get_all(self) -> Dict[str, str]:
        virt_info: Dict[str, str] = {}
        firmware_info: Dict[str, str] = self.get_firmware_info()

        virt_info.update(firmware_info)
        return virt_info


# TODO/FIXME: As a first pass, move dmi and the generic firmware code here,
#             even though with kernels with sysfs dmi support, and a recent
#             version of dmidecode (> 3.0), most of the dmi info is readable
#             by a user. However, the system-uuid is not readable by a user,
#             and that is pretty much the only thing from DMI we care about,
def get_firmware_collector(
    arch,
    prefix: str = None,
    testing: bool = None,
    collected_hw_info: Dict[str, Union[str, int, bool, None]] = None,
) -> Union[NullFirmwareInfoCollector, dmiinfo.DmidecodeFactCollector]:
    """
    Return a class that can be used to get firmware info specific to
    this systems platform.

    ie, DmiFirmwareInfoProvider on intel platforms, and a
    NullFirmwareInfoProvider otherwise.
    """
    # we could potentially consider /proc/sysinfo as a FirmwareInfoProvider
    # but at the moment, it is just firmware/dmi stuff.

    if arch in ARCHES_WITHOUT_DMI:
        log.debug("Not looking for DMI info since it is not available on '%s'" % arch)
        firmware_provider_class = NullFirmwareInfoCollector
    else:
        firmware_provider_class = dmiinfo.DmidecodeFactCollector

    firmware_provider: Union[NullFirmwareInfoCollector, dmiinfo.DmidecodeFactCollector]
    firmware_provider = firmware_provider_class(
        prefix=prefix, testing=testing, collected_hw_info=collected_hw_info
    )

    return firmware_provider



# Copyright (c) 2016 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
import json
import logging
import threading
import dbus
import dbus.service
import subscription_manager.injection as inj

from rhsmlib.dbus import constants, exceptions, dbus_utils, base_object, server, util
from rhsmlib.services.register import RegisterService
from rhsmlib.services.unregister import UnregisterService
from rhsmlib.services.attach import AttachService
from rhsmlib.services.entitlement import EntitlementService
from rhsmlib.client_info import DBusSender

from subscription_manager.i18n import Locale
from subscription_manager.i18n import ugettext as _
from subscription_manager.entcertlib import EntCertActionInvoker

log = logging.getLogger(__name__)


class RegisterDBusObject(base_object.BaseObject):
    default_dbus_path = constants.REGISTER_DBUS_PATH
    interface_name = constants.REGISTER_INTERFACE

    def __init__(self, conn=None, object_path=None, bus_name=None):
        super(RegisterDBusObject, self).__init__(conn=conn, object_path=object_path, bus_name=bus_name)
        self.started_event = threading.Event()
        self.stopped_event = threading.Event()
        self.server = None
        self.lock = threading.Lock()

    @util.dbus_service_method(
        constants.REGISTER_INTERFACE,
        in_signature="s",
        out_signature="s",
    )
    @util.dbus_handle_sender
    @util.dbus_handle_exceptions
    def Start(self, locale, sender=None):
        locale = dbus_utils.dbus_to_python(locale, expected_type=str)
        Locale.set(locale)

        with self.lock:
            if self.server:
                return self.server.address

            log.debug("Attempting to create new domain socket server")
            cmd_line = DBusSender.get_cmd_line(sender)
            self.server = server.DomainSocketServer(
                object_classes=[DomainSocketRegisterDBusObject], sender=sender, cmd_line=cmd_line
            )
            address = self.server.run()
            log.debug('DomainSocketServer for sender %s created and listening on "%s"' % (sender, address))
            return address

    @util.dbus_service_method(
        constants.REGISTER_INTERFACE,
        in_signature="s",
        out_signature="b",
    )
    @util.dbus_handle_sender
    @util.dbus_handle_exceptions
    def Stop(self, locale, sender=None):
        locale = dbus_utils.dbus_to_python(locale, expected_type=str)
        Locale.set(locale)
        with self.lock:
            if self.server:
                self.server.shutdown()
                self.server = None
                log.debug("Stopped DomainSocketServer")
                return True
            else:
                raise exceptions.Failed("No domain socket server is running")


class OrgNotSpecifiedException(dbus.DBusException):
    """
    This exception is intended for signaling that user is member of more
    organizations and no organization was specified
    """

    _dbus_error_name = "%s.Error" % constants.REGISTER_INTERFACE
    include_traceback = False
    severity = "info"

    def __init__(self, username):
        self.username = username

    def __str__(self):
        return _("User %s is member of more organizations, but no organization was selected") % self.username


class NoOrganizationException(dbus.DBusException):
    """
    This exception is intended for signaling that user is not member of any
    organization and such user cannot register to candlepin server.
    """

    _dbus_error_name = "%s.Error" % constants.REGISTER_INTERFACE
    include_traceback = False

    def __init__(self, username):
        self.username = username

    def __str__(self):
        return _("User %s is not member of any organization") % self.username


class DomainSocketRegisterDBusObject(base_object.BaseObject):
    interface_name = constants.PRIVATE_REGISTER_INTERFACE
    default_dbus_path = constants.PRIVATE_REGISTER_DBUS_PATH

    def __init__(self, conn=None, object_path=None, bus_name=None, sender=None, cmd_line=None):
        # On our DomainSocket DBus server since a private connection is not a "bus", we have to treat
        # it slightly differently. In particular there are no names, no discovery and so on.
        super(DomainSocketRegisterDBusObject, self).__init__(
            conn=conn, object_path=object_path, bus_name=bus_name
        )
        self.sender = sender
        self.cmd_line = cmd_line

    @dbus.service.method(
        dbus_interface=constants.PRIVATE_REGISTER_INTERFACE,
        in_signature="ssa{sv}s",
        out_signature="s",
    )
    @util.dbus_handle_exceptions
    def GetOrgs(self, username, password, connection_options, locale):
        """
        This method tries to return list of organization user belongs to. This method also uses
        basic authentication method (using username and password).

        :param username: string with username used for connection to candlepin server
        :param password: string with password
        :param connection_options: dictionary with connection options
        :param locale: string with locale
        :return: string with json returned by candlepin server
        """
        connection_options = dbus_utils.dbus_to_python(connection_options, expected_type=dict)
        connection_options["username"] = dbus_utils.dbus_to_python(username, expected_type=str)
        connection_options["password"] = dbus_utils.dbus_to_python(password, expected_type=str)
        locale = dbus_utils.dbus_to_python(locale, expected_type=str)

        with DBusSender() as dbus_sender:
            dbus_sender.set_cmd_line(sender=self.sender, cmd_line=self.cmd_line)
            Locale.set(locale)
            cp = self.build_uep(connection_options, basic_auth_method=True)
            owners = cp.getOwnerList(connection_options["username"])
            dbus_sender.reset_cmd_line()

        return json.dumps(owners)

    @util.dbus_service_signal(
        constants.PRIVATE_REGISTER_INTERFACE,
        signature="s",
    )
    @util.dbus_handle_exceptions
    def UserMemberOfOrgs(self, orgs):
        """
        Signal triggered, when user tries to register, no organization is specified, but
        user is member of more than one organization and it will be necessary to select
        one organization in client application consuming this D-Bus API
        :param orgs: string with json.dump of org dictionary
        :return: None
        """
        log.debug(
            "D-Bus signal UserMemberOfOrgs emitted on the interface %s with arg: %s"
            % (constants.PRIVATE_REGISTER_INTERFACE, orgs)
        )
        return None

    def _no_owner_cb(self, username):
        """
        Callback method that is triggered, when given user is not member of any organization.
        In this case exception is raised.
        :return: None
        """
        raise NoOrganizationException(username=username)

    def _get_owner_cb(self, owners):
        """
        When there is necessary to select one organization by user, then signal is triggered.
        :param owners: The list of owner objects
        :return: None
        """
        # We use string of json.dumped dictionary, because D-Bus API does not work
        # properly with dictionaries
        orgs = json.dumps(owners)

        # NOTE: use only position argument. Do not change it to keyed argument, because
        # D-Bus wrapper does NOT work with keyed argument
        self.UserMemberOfOrgs(orgs)

        # We return None here, because we cannot know what will be selected by user
        return None

    @staticmethod
    def _enable_content(cp, consumer: dict) -> None:
        """
        Try to enable content. Try to do auto-attach in non-SCA mode or try to do refresh SCA mode.
        :param cp: Object representing connection to candlepin server
        :param consumer: Dictionary representing consumer
        :return: None
        """
        content_access_mode = consumer["owner"]["contentAccessMode"]
        enabled_content = None

        if content_access_mode == "entitlement":
            log.debug("Auto-attaching due to enable_content option")
            attach_service = AttachService(cp)
            enabled_content = attach_service.attach_auto()
            if len(enabled_content) > 0:
                log.debug("Updating entitlement certificates")
                # FIXME: The enabled_content contains all data necessary for generating entitlement
                # certificate and private key. Thus we could save few REST API calls, when the data was used.
                EntCertActionInvoker().update()
            else:
                log.debug("Skipping updating entitlement certificates, because no content was enabled")
        elif content_access_mode == "org_environment":
            log.debug("Refreshing due to enabled_content option and simple content access mode")
            entitlement_service = EntitlementService(cp)
            # TODO: try get anything useful from refresh result. It is not possible atm.
            entitlement_service.refresh(remove_cache=False, force=False)
        else:
            log.error(
                f"Unable to enable content due to unsupported content access mode: " f"{content_access_mode}"
            )

        # When it was possible to enable content, then extend consumer
        # with information about enabled content
        if enabled_content is not None:
            consumer["enabledContent"] = enabled_content

    @staticmethod
    def _remove_enable_content_option(options: dict) -> bool:
        """
        Try to remove enable_content option from options used in Register() and
        RegisterWithActivationKeys() methods
        :param options: dictionary with options
        :return: Value of enable_options
        """
        if "enable_content" in options:
            log.debug(
                f"Value and type of enable_content: '{options['enable_content']}' "
                f"({type(options['enable_content'])})"
            )
            enable_content = bool(options.pop("enable_content"))
            log.debug(f"Value of converted enable_content: {enable_content}")
        else:
            enable_content = False
        return enable_content

    def _unregister(self, connection_options: dict) -> None:
        """
        Unregisters a system that is currently registered and
        cleans the cp_provider to handle authorization after un-registering.
        :param connection_options: dictionary with connection options
        :return: None
        """
        self.ensure_registered()
        log.info("This system is already registered, attempting to un-register...")

        cp_provider = inj.require(inj.CP_PROVIDER)

        cp = self.build_uep(connection_options)
        UnregisterService(cp).unregister()

        # The CPProvider object must be cleaned and the cp object must
        # be re-initialized to handle authorization after un-registration.
        cp_provider.clean()

    @dbus.service.method(
        dbus_interface=constants.PRIVATE_REGISTER_INTERFACE,
        in_signature="sssa{sv}a{sv}s",
        out_signature="s",
    )
    @util.dbus_handle_exceptions
    def Register(self, org, username, password, options, connection_options, locale):
        """
        This method registers the system using basic auth
        (username and password for a given org).
        For any option that is required but not included the default will be
        used.

        Options is a dict of strings that modify the outcome of this method.

        Note this method is registration ONLY.  Auto-attach is a separate process.
        """
        org = dbus_utils.dbus_to_python(org, expected_type=str)
        connection_options = dbus_utils.dbus_to_python(connection_options, expected_type=dict)
        connection_options["username"] = dbus_utils.dbus_to_python(username, expected_type=str)
        connection_options["password"] = dbus_utils.dbus_to_python(password, expected_type=str)
        options = dbus_utils.dbus_to_python(options, expected_type=dict)
        locale = dbus_utils.dbus_to_python(locale, expected_type=str)

        force_registration: bool = options.get("force", False)
        system_is_registered: bool = self.is_registered()
        if system_is_registered and not force_registration:
            raise dbus.DBusException("This system is already registered.")

        with DBusSender() as dbus_sender:
            dbus_sender.set_cmd_line(sender=self.sender, cmd_line=self.cmd_line)
            Locale.set(locale)

            # If the system is registered and the 'force' option is specified,
            # Unregister the system and register the system again.
            if system_is_registered and force_registration:
                self._unregister(connection_options)
            cp = self.build_uep(connection_options)

            register_service = RegisterService(cp)

            # Try to get organization from the list available organizations, when the list contains
            # only one item, then register_service.determine_owner_key will return this organization
            if not org:
                org = register_service.determine_owner_key(
                    username=connection_options["username"],
                    get_owner_cb=self._get_owner_cb,
                    no_owner_cb=self._no_owner_cb,
                )

            # When there is more organizations, then signal was triggered in callback method
            # _get_owner_cb, but some exception has to be raised here to not try registration process
            if not org:
                raise OrgNotSpecifiedException(username=connection_options["username"])

            # Remove 'enable_content' option, because it will not be proceed in register service
            enable_content = self._remove_enable_content_option(options)

            consumer = register_service.register(org, **options)

            # When consumer is created, then we can try to enabled content, when it was
            # requested in options.
            if enable_content is True:
                self._enable_content(cp, consumer)

            dbus_sender.reset_cmd_line()

        return json.dumps(consumer)

    @dbus.service.method(
        dbus_interface=constants.PRIVATE_REGISTER_INTERFACE,
        in_signature="sasa{sv}a{sv}s",
        out_signature="s",
    )
    @util.dbus_handle_exceptions
    def RegisterWithActivationKeys(self, org, activation_keys, options, connection_options, locale):
        """
        Note this method is registration ONLY.  Auto-attach is a separate process.
        """
        connection_options = dbus_utils.dbus_to_python(connection_options, expected_type=dict)
        options = dbus_utils.dbus_to_python(options, expected_type=dict)
        options["activation_keys"] = dbus_utils.dbus_to_python(activation_keys, expected_type=list)
        org = dbus_utils.dbus_to_python(org, expected_type=str)
        locale = dbus_utils.dbus_to_python(locale, expected_type=str)

        force_registration: bool = options.get("force", False)
        system_is_registered: bool = self.is_registered()
        if system_is_registered and not force_registration:
            raise dbus.DBusException("This system is already registered.")

        with DBusSender() as dbus_sender:
            dbus_sender.set_cmd_line(sender=self.sender, cmd_line=self.cmd_line)
            Locale.set(locale)

            # If the system is registered and the 'force' option is specified,
            # Unregister the system and register the system again.
            if system_is_registered and force_registration:
                self._unregister(connection_options)
            cp = self.build_uep(connection_options)

            register_service = RegisterService(cp)
            consumer = register_service.register(org, **options)

            log.debug("System registered, updating entitlements if needed")
            ent_cert_lib = EntCertActionInvoker()
            ent_cert_lib.update()

            dbus_sender.reset_cmd_line()

        return json.dumps(consumer)



#
# Subscription manager command line utility.
#
# Copyright (c) 2021 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
import os

import rhsm.connection as connection
import subscription_manager.injection as inj

from subscription_manager.cli import system_exit
from subscription_manager.cli_command.cli import CliCommand, handle_exception
from subscription_manager.i18n import ugettext as _


class RedeemCommand(CliCommand):
    def __init__(self):
        shortdesc = _("Attempt to redeem a subscription for a preconfigured system")
        super(RedeemCommand, self).__init__("redeem", shortdesc, False)

        self.parser.add_argument(
            "--email",
            dest="email",
            action="store",
            help=_("email address to notify when " "subscription redemption is complete"),
        )
        self.parser.add_argument(
            "--locale",
            dest="locale",
            action="store",
            help=_(
                "optional language to use for email "
                "notification when subscription redemption is "
                "complete (Examples: en-us, de-de)"
            ),
        )

    def _validate_options(self):
        if not self.options.email:
            system_exit(
                os.EX_USAGE, _("Error: This command requires that you specify an email address with --email.")
            )

    def _do_command(self):
        """
        Executes the command.
        """
        self.assert_should_be_registered()
        self._validate_options()

        try:
            # FIXME: why just facts and package profile update here?
            # update facts first, if we need to
            facts = inj.require(inj.FACTS)
            facts.update_check(self.cp, self.identity.uuid)

            profile_mgr = inj.require(inj.PROFILE_MANAGER)
            profile_mgr.update_check(self.cp, self.identity.uuid)

            # BZ 1248833 Ensure we print out the display message if we get any back
            response = self.cp.activateMachine(self.identity.uuid, self.options.email, self.options.locale)
            if response is None:
                system_exit(os.EX_SOFTWARE, _("Error: Unable to redeem subscription for this system."))
            if response.get("displayMessage"):
                system_exit(0, response.get("displayMessage"))
        except connection.GoneException as ge:
            raise ge
        except connection.RestlibException as e:
            # candlepin throws an exception during activateMachine, even for
            # 200's. We need to look at the code in the RestlibException and proceed
            # accordingly
            if 200 <= e.code <= 210:
                system_exit(0, e)
            else:
                handle_exception("Unable to redeem: {e}".format(e=e), e)
        except Exception as e:
            handle_exception("Unable to redeem: {e}".format(e=e), e)

        self._request_validity_check()



from ..test_managercli import TestCliProxyCommand
from subscription_manager import managercli
from subscription_manager.cli_command import release

from unittest.mock import patch, Mock


class TestReleaseCommand(TestCliProxyCommand):
    command_class = managercli.ReleaseCommand

    def test_main_proxy_url_release(self):
        proxy_host = "example.com"
        proxy_port = "3128"
        proxy_url = "%s:%s" % (proxy_host, proxy_port)

        with patch.object(managercli.ReleaseCommand, "_get_consumer_release"):
            self.cc.main(["--proxy", proxy_url])
            self._orig_do_command()

            # FIXME: too many stubs atm to make this meaningful
            # self.assertEquals(proxy_host, self.cc.cp_provider.content_connection.proxy_hostname)

            self.assertEqual(proxy_url, self.cc.options.proxy_url)
            self.assertEqual(type(proxy_url), type(self.cc.options.proxy_url))
            self.assertEqual(proxy_host, self.cc.proxy_hostname)
            self.assertEqual(int(proxy_port), self.cc.proxy_port)

    def test_release_set_updates_repos(self):
        mock_repo_invoker = Mock()
        with patch.object(release, "RepoActionInvoker", Mock(return_value=mock_repo_invoker)):
            with patch.object(release.ReleaseBackend, "get_releases", Mock(return_value=["7.2"])):
                with patch.object(managercli.ReleaseCommand, "_get_consumer_release"):
                    self.cc.main(["--set=7.2"])
                    self._orig_do_command()

                    mock_repo_invoker.update.assert_called_with()

    def test_release_unset_updates_repos(self):
        mock_repo_invoker = Mock()
        with patch.object(release, "RepoActionInvoker", Mock(return_value=mock_repo_invoker)):
            with patch.object(managercli.ReleaseCommand, "_get_consumer_release"):
                self.cc.main(["--unset"])
                self._orig_do_command()

                mock_repo_invoker.update.assert_called_with()



# Copyright (c) 2010 - 2012 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
import base64
import os

from rhsm import certificate, _certificate
from rhsm.certificate2 import EntitlementCertificate

from rct.commands import RCTCliCommand
from rct.printing import printc, type_to_string

from subscription_manager.cli import InvalidCLIOptionError

from subscription_manager.i18n import ugettext as _


class RCTCertCommand(RCTCliCommand):
    def __init__(self, name="cli", aliases=None, shortdesc=None, primary=False):
        RCTCliCommand.__init__(self, name=name, aliases=aliases, shortdesc=shortdesc, primary=primary)

    def _get_usage(self):
        return _("%(prog)s {name} [OPTIONS] CERT_FILE").format(name=self.name)

    def _create_cert(self):
        cert_file = self._get_file_from_args()
        try:
            return certificate.create_from_file(cert_file)
        except certificate.CertificateException as ce:
            raise InvalidCLIOptionError(
                _("Unable to read certificate file '{certificate}': {exception}").format(
                    certificate=cert_file, exception=ce
                )
            )

    def _validate_options(self):
        cert_file = self._get_file_from_args()
        if not cert_file:
            raise InvalidCLIOptionError(_("You must specify a certificate file."))

        if not os.path.isfile(cert_file):
            raise InvalidCLIOptionError(_("The specified certificate file does not exist."))


class CatCertCommand(RCTCertCommand):
    def __init__(self):
        RCTCliCommand.__init__(
            self,
            name="cat-cert",
            aliases=["cc"],
            shortdesc=_("Print certificate information"),
            primary=True,
        )

        self.parser.add_argument(
            "--no-products",
            dest="no_products",
            action="store_true",
            help=_("do not show the cert's product information"),
        )
        self.parser.add_argument(
            "--no-content",
            dest="no_content",
            action="store_true",
            help=_("do not show the cert's content info"),
        )

    def _do_command(self):
        """
        Does the work that this command intends.
        """
        cert = self._create_cert()
        printc(cert, skip_content=self.options.no_content, skip_products=self.options.no_products)


class StatCertCommand(RCTCertCommand):
    def __init__(self):
        RCTCliCommand.__init__(
            self,
            name="stat-cert",
            aliases=["sc"],
            shortdesc=_("Print certificate statistics and sizes"),
            primary=True,
        )

    def _do_command(self):
        cert = self._create_cert()
        pem = self._get_pem(self._get_file_from_args())
        print(_("Type: {certificate_type}").format(certificate_type=type_to_string(cert)))
        print(_("Version: {certificate_version}").format(certificate_version=cert.version))
        print(_("DER size: {size}b").format(size=self._get_der_size(pem)))

        subject_key_id = self._get_subject_key_id(pem)
        if subject_key_id is not None:
            print(
                _("Subject Key ID size: {len_subject_key_id}b").format(len_subject_key_id=len(subject_key_id))
            )

        if isinstance(cert, EntitlementCertificate):
            content_len = 0
            if cert.content:
                content_len = len(cert.content)
            print(_("Content sets: {content_length}").format(content_length=content_len))

    def _get_pem(self, filename):
        return open(filename, "r").read()

    def _get_der_size(self, pem):
        parts = pem.split("-----BEGIN CERTIFICATE-----\n")
        cert = parts[1].split("-----END CERTIFICATE-----")[0]
        return len(base64.b64decode(cert))

    def _get_subject_key_id(self, pem):
        cert = _certificate.load(pem=pem)
        subject_key_id = cert.get_extension(oid="2.5.29.14")
        return subject_key_id



#
# Subscription manager command line utility.
#
# Copyright (c) 2021 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
import logging
import os

import rhsm.connection as connection

from subscription_manager.cli import system_exit
from subscription_manager.cli_command.cli import handle_exception
from subscription_manager.cli_command.list import ORG_LIST
from subscription_manager.cli_command.user_pass import UserPassCommand
from subscription_manager.i18n import ugettext as _
from subscription_manager.printing_utils import columnize, echo_columnize_callback

log = logging.getLogger(__name__)


class OwnersCommand(UserPassCommand):
    def __init__(self):
        shortdesc = _("Display the organizations against which a user can register a system")

        super(OwnersCommand, self).__init__("orgs", shortdesc, False)

        self._add_url_options()

    def _do_command(self):

        try:
            # get a UEP
            if self.options.token:
                self.cp = self.cp_provider.get_keycloak_auth_cp(self.options.token)
            else:
                self.cp_provider.set_user_pass(self.username, self.password)
                self.cp = self.cp_provider.get_basic_auth_cp()
            owners = self.cp.getOwnerList(self.username)
            log.debug("Successfully retrieved org list from server.")
            if len(owners):
                print("+-------------------------------------------+")
                print("          {name} {label}".format(name=self.username, label=_("Organizations")))
                print("+-------------------------------------------+")
                print("")
                for owner in owners:
                    print(
                        columnize(ORG_LIST, echo_columnize_callback, owner["displayName"], owner["key"])
                        + "\n"
                    )
            else:
                print(_("{username} cannot register with any organizations.").format(username=self.username))

        except connection.RestlibException as re:
            log.exception(re)
            log.error("Error: Unable to retrieve org list from server: {re}".format(re=re))

            system_exit(os.EX_SOFTWARE, re)
        except Exception as e:
            handle_exception(_("Error: Unable to retrieve org list from server"), e)



from ..fixture import SubManFixture
from ..test_managercli import TestCliProxyCommand
from unittest.mock import Mock
from subscription_manager import managercli
from subscription_manager.cache import ContentAccessCache, ContentAccessModeCache
from subscription_manager.injection import provide, CONTENT_ACCESS_CACHE, CONTENT_ACCESS_MODE_CACHE


class TestRefreshCommand(TestCliProxyCommand):
    command_class = managercli.RefreshCommand

    def test_force_option(self):
        self.cc.main(["--force"])


class TestRefreshCommandWithDoCommand(SubManFixture):
    command_class = managercli.RefreshCommand

    def setUp(self):
        super(TestRefreshCommandWithDoCommand, self).setUp()
        self.cc = self.command_class()

    def test_cache_removed(self):
        # lots of mocking basically to show that the injected content access
        # cache and content access mode caches are cleared on each run of the
        # refresh command
        self.cc.assert_should_be_registered = Mock(return_value=True)
        mock_content_access_cache = Mock(spec=ContentAccessCache)
        mock_content_access_cache.return_value.exists.return_value = True
        provide(CONTENT_ACCESS_CACHE, mock_content_access_cache)
        mock_content_access_mode_cache = Mock(spec=ContentAccessModeCache)
        mock_content_access_mode_cache.return_value.exists.return_value = True
        provide(CONTENT_ACCESS_MODE_CACHE, mock_content_access_mode_cache)

        self.cc.main([])

        # This cache should not be deleted to be able to use HTTP header 'If-Modified-Since'
        mock_content_access_cache.return_value.remove.assert_not_called()
        # Cache about content access mode should be deleted, because content access mode
        # can be changed from SCA to entitlement and vice versa
        mock_content_access_mode_cache.return_value.exists.assert_called_once()
        mock_content_access_mode_cache.return_value.delete_cache.assert_called_once()



# Copyright (c) 2016 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
import logging
from typing import Dict

from rhsmlib.facts import collector

log = logging.getLogger(__name__)


class CleanupCollector(collector.FactsCollector):
    def get_all(self) -> Dict[str, int]:
        cleanup_facts: Dict[str, int] = {}
        dmi_socket_info = self.replace_socket_count_with_dmi()
        cleanup_facts.update(dmi_socket_info)
        return cleanup_facts

    def replace_socket_count_with_dmi(self) -> Dict[str, int]:
        cleanup_info: Dict[str, int] = {}
        # cpu topology reporting on xen dom0 machines is wrong. So
        # if we are a xen dom0, and we found socket info in dmiinfo,
        # replace our normal cpu socket calculation with the dmiinfo one
        # we have to do it after the virt data and cpu data collection
        if "virt.host_type" not in self._collected_hw_info:
            return cleanup_info

        if not self._host_is_xen_dom0():
            return cleanup_info

        if "dmi.meta.cpu_socket_count" not in self._collected_hw_info:
            return cleanup_info

        # Alright, lets munge up cpu socket info based on the dmi info.
        socket_count = int(self._collected_hw_info["dmi.meta.cpu_socket_count"])
        cleanup_info["cpu.cpu_socket(s)"] = socket_count

        if "cpu.cpu(s)" not in self._collected_hw_info:
            return cleanup_info

        # And the cores per socket count as well
        dmi_cpu_cores_per_cpu = int(self._collected_hw_info["cpu.cpu(s)"]) // socket_count
        cleanup_info["cpu.core(s)_per_socket"] = dmi_cpu_cores_per_cpu

        return cleanup_info

    def _host_is_xen_dom0(self) -> bool:
        return self._collected_hw_info["virt.host_type"].find("dom0") > -1



import os
import logging

from subscription_manager import cpuinfo

from . import fixture

log = logging.getLogger("rhsm-app.unittests." + __name__)

test_path = os.path.dirname(__file__)
CPUINFOS = "data/cpuinfo"

cpu_data_dir = os.path.join(test_path, CPUINFOS)


class BaseCpuInfo(fixture.SubManFixture):
    expected = {}
    cpuinfo_class = None

    def _test(self, name):
        cpud = self._load_cpuinfo(name)
        exp = self.expected[name]

        x = self.cpuinfo_class.from_proc_cpuinfo_string(cpud)

        if "cpu_count" in exp:
            self.assertEqual(exp["cpu_count"], len(x.cpu_info.processors))
        if "model" in exp:
            self.assertEqual(exp["model"], x.cpu_info.model)
        if "machine" in exp:
            self.assertEqual(exp["machine"], x.cpu_info.common["machine"])

    def _load_cpuinfo(self, name):
        f = open(os.path.join(cpu_data_dir, name), "r")
        cpuinfo_data = f.read()
        return cpuinfo_data

    def _test_fact_key(self, key):
        bad_letters = " .   "

        for bad_letter in bad_letters:
            self.assertFalse(bad_letter in key)


class TestX86_64CpuInfo(BaseCpuInfo):
    expected = {
        "x86_64-dell-e4310-1socket-2core-4cpu": {"cpu_count": 4, "model": "37"},
        "x86_64-dell-t7600-2sockets-6core-24cpu": {"cpu_count": 24, "model": "45"},
        "x86_64-4socket-8core-64cpu": {"cpu_count": 64},
        "armv7-samsung-1socket-2core-2cpu": {},
    }
    cpuinfo_class = cpuinfo.X86_64CpuInfo

    def test_x86_64_dell_e4310(self):
        self._test("x86_64-dell-e4310-1socket-2core-4cpu")

    def test_x86_64_64cpu(self):
        self._test("x86_64-4socket-8core-64cpu")

    def test_x86_64_24cpu_dell_t7600(self):
        self._test("x86_64-dell-t7600-2sockets-6core-24cpu")

    def test_armv7(self):
        # verify we fail nicely when given a completely different /proc/cpuinfo
        self._test("armv7-samsung-1socket-2core-2cpu")


class TestPpc64CpuInfo(BaseCpuInfo):
    cpuinfo_class = cpuinfo.Ppc64CpuInfo
    expected = {
        "ppc64-power8-16cpu-kvm": {
            "cpu_count": 16,
            "platform": "pSeries",
            "model": "IBM pSeries (emulated by qemu)",
        },
        "ppc64-power8-160cpu-powernv": {"cpu_count": 160, "platform": "powerNV", "model": "8247-22L"},
    }

    def test_ppc64_power8_160cpu_powernv(self):
        self._test("ppc64-power8-160cpu-powernv")

    def test_ppc64_power8_16cpu_kvm(self):
        self._test("ppc64-power8-16cpu-kvm")


class TestPpc64leCpuInfo(BaseCpuInfo):
    cpuinfo_class = cpuinfo.Ppc64CpuInfo
    expected = {
        "ppc64le-power8-16cpu-lpar": {
            "cpu_count": 16,
            "platform": "pSeries",
            "model": "IBM,8247-22L",
            "machine": "CHRP IBM,8247-22L",
        }
    }

    def test_ppc64le_power8_16cpu_lpar(self):
        self._test("ppc64le-power8-16cpu-lpar")


class TestAarch64CpuInfo(BaseCpuInfo):
    cpuinfo_class = cpuinfo.Aarch64CpuInfo
    expected = {
        "aarch64-mustang-dev-rhel7-1socket-8core-8cpu": {
            "cpu_count": 8,
            "model_name": "AArch64 Processor rev 0 (aarch64)",
        },
        "aarch64-hp-moonshot-1socket-8core-8cpu": {"cpu_count": 8, "model_name": None},
    }

    def test_mustang(self):
        self._test("aarch64-mustang-dev-rhel7-1socket-8core-8cpu")

    def test_moonshot(self):
        self._test("aarch64-hp-moonshot-1socket-8core-8cpu")



# Copyright (c) 2016 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
import unittest

from rhsm.config import RhsmConfigParser, NoOptionError
from rhsmlib.services.config import Config, ConfigSection
from test.rhsmlib.base import TestUtilsMixin

TEST_CONFIG = """
[foo]
bar =
quux = baz
bigger_than_32_bit = 21474836470
bigger_than_64_bit = 123456789009876543211234567890

[server]
hostname = server.example.com
prefix = /candlepin
port = 8443
insecure = 1
proxy_hostname =
proxy_port =
proxy_user =
proxy_password =

[rhsm]
ca_cert_dir = /etc/rhsm/ca-test/
baseurl = https://content.example.com
repomd_gpg_url =
repo_ca_cert = %(ca_cert_dir)sredhat-uep-non-default.pem
productCertDir = /etc/pki/product
entitlementCertDir = /etc/pki/entitlement
consumerCertDir = /etc/pki/consumer
report_package_profile = 1
pluginDir = /usr/lib/rhsm-plugins
some_option = %(repo_ca_cert)stest
manage_repos =

[rhsmcertd]
certCheckInterval = 245

[logging]
default_log_level = DEBUG
"""


class BaseConfigTest(unittest.TestCase, TestUtilsMixin):
    expected_sections = ["foo", "server", "rhsm", "rhsmcertd", "logging"]

    def setUp(self):
        super(BaseConfigTest, self).setUp()
        self.fid = self.write_temp_file(TEST_CONFIG)
        self.parser = RhsmConfigParser(self.fid.name)
        self.config = Config(self.parser)
        self.addCleanup(self.fid.close)


class TestConfig(BaseConfigTest):
    def test_config_contains(self):
        self.assertTrue("server" in self.config)
        self.assertFalse("not_here" in self.config)

    def test_config_len(self):
        self.assertEqual(len(self.expected_sections), len(self.config))

    def test_keys(self):
        self.assert_items_equals(self.expected_sections, list(self.config.keys()))

    def test_values(self):
        values = list(self.config.values())
        for v in values:
            self.assertIsInstance(v, ConfigSection)

    def test_set_new_section(self):
        self.config["new_section"] = {"hello": "world"}
        self.assertEqual(["hello"], self.config._parser.options("new_section"))
        self.assertEqual("world", self.config._parser.get("new_section", "hello"))

    def test_set_old_section(self):
        self.config["foo"] = {"hello": "world"}
        self.assertEqual(["hello"], self.config._parser.options("foo"))
        self.assertEqual("world", self.config._parser.get("foo", "hello"))
        self.assertRaises(NoOptionError, self.config._parser.get, "foo", "quux")

    def test_get_item(self):
        self.assertIsInstance(self.config["server"], ConfigSection)

    def test_persist(self):
        self.config["foo"] = {"hello": "world"}
        self.config.persist()
        reparsed = RhsmConfigParser(self.fid.name)
        self.assertEqual("world", reparsed.get("foo", "hello"))
        self.assertRaises(NoOptionError, reparsed.get, "foo", "quux")

    def test_auto_persists(self):
        config = Config(self.parser, auto_persist=True)
        config["foo"] = {"hello": "world"}
        reparsed = RhsmConfigParser(self.fid.name)
        self.assertEqual("world", reparsed.get("foo", "hello"))
        self.assertRaises(NoOptionError, reparsed.get, "foo", "quux")

    def test_does_not_auto_persist_by_default(self):
        config = Config(self.parser, auto_persist=False)
        config["foo"] = {"hello": "world"}
        reparsed = RhsmConfigParser(self.fid.name)
        self.assertEqual("baz", reparsed.get("foo", "quux"))
        self.assertRaises(NoOptionError, reparsed.get, "foo", "hello")

    def test_del_item(self):
        del self.config["foo"]
        self.assertFalse(self.config._parser.has_section("foo"))

    def test_iter(self):
        sections = [s for s in self.config]
        self.assert_items_equals(self.expected_sections, sections)


class TestConfigSection(BaseConfigTest):
    def test_get_value(self):
        self.assertEqual("1", self.config["server"]["insecure"])

    def test_get_missing_value(self):
        with self.assertRaises(KeyError):
            self.config["server"]["missing"]

    def test_set_item(self):
        self.assertEqual("baz", self.config["foo"]["quux"])
        self.config["foo"]["quux"] = "fizz"
        self.assertEqual("fizz", self.config["foo"]["quux"])

    def test_auto_persist(self):
        config = Config(self.parser, auto_persist=True)
        self.assertEqual("baz", config["foo"]["quux"])
        config["foo"]["quux"] = "fizz"
        self.assertEqual("fizz", config["foo"]["quux"])

        reparsed = RhsmConfigParser(self.fid.name)
        self.assertEqual("fizz", reparsed.get("foo", "quux"))

    def test_persist_cascades(self):
        config = Config(self.parser, auto_persist=False)
        self.assertEqual("baz", config["foo"]["quux"])
        config["foo"]["quux"] = "fizz"
        config.persist()
        self.assertEqual("fizz", config["foo"]["quux"])

        reparsed = RhsmConfigParser(self.fid.name)
        self.assertEqual("fizz", reparsed.get("foo", "quux"))

    def test_del_item(self):
        del self.config["foo"]["quux"]
        self.assertNotIn("quux", self.config["foo"])

        with self.assertRaises(KeyError):
            del self.config["foo"]["missing_key"]

    def test_len(self):
        self.assertEqual(4, len(self.config["foo"]))

    def test_in(self):
        self.assertIn("quux", self.config["foo"])
        self.assertNotIn("missing", self.config["foo"])



# Copyright (c) 2017 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.

from rhsmlib.dbus.objects.consumer import ConsumerDBusObject

from unittest import mock
from test.rhsmlib.base import DBusServerStubProvider


class TestConsumerDBusObject(DBusServerStubProvider):
    dbus_class = ConsumerDBusObject
    dbus_class_kwargs = {}

    @classmethod
    def setUpClass(cls) -> None:
        get_consumer_uuid_patch = mock.patch(
            "rhsmlib.dbus.objects.consumer.Consumer.get_consumer_uuid",
            name="get_consumer_uuid",
        )
        cls.patches["get_consumer_uuid"] = get_consumer_uuid_patch.start()
        cls.addClassCleanup(get_consumer_uuid_patch)

        super().setUpClass()

    def test_GetUuid(self):
        self.patches["get_consumer_uuid"].return_value = "fake-uuid"

        expected = "fake-uuid"
        result = self.obj.GetUuid.__wrapped__(self.obj, self.LOCALE)
        self.assertEqual(expected, result)
